@@ -134,31 +134,17 @@ QT_BEGIN_NAMESPACE
     \section2 Example Usage
 
     The following snippet shows a simple Map and the necessary Plugin type
-    to use it. The map is centered near Brisbane, Australia, zoomed out to the
-    minimum zoom level, with gesture interaction enabled.
+    to use it. The map is centered over Oslo, Norway, with zoom level 10.
 
-    \code
-    Plugin {
-        id: somePlugin
-        // code here to choose the plugin as necessary
-    }
+    \quotefromfile minimal_map/main.qml
+    \skipto import
+    \printuntil }
+    \printline }
+    \skipto Map
+    \printuntil }
+    \printline }
 
-    Map {
-        id: map
-
-        plugin: somePlugin
-
-        center {
-            latitude: -27
-            longitude: 153
-        }
-        zoomLevel: map.minimumZoomLevel
-
-        gesture.enabled: true
-    }
-    \endcode
-
-    \image api-map.png
+    \image minimal_map.png
 */
 
 /*!
@@ -168,6 +154,8 @@ QT_BEGIN_NAMESPACE
     application should open the link in a browser or display its contents to the user.
 */
 
+static const qreal EARTH_MEAN_RADIUS = 6371007.2;
+
 QDeclarativeGeoMap::QDeclarativeGeoMap(QQuickItem *parent)
         : QQuickItem(parent),
         m_plugin(0),
@@ -175,13 +163,15 @@ QDeclarativeGeoMap::QDeclarativeGeoMap(QQuickItem *parent)
         m_mappingManager(0),
         m_center(51.5073,-0.1277), //London city center
         m_activeMapType(0),
-        m_gestureArea(0),
+        m_gestureArea(new QQuickGeoMapGestureArea(this)),
         m_map(0),
         m_error(QGeoServiceProvider::NoError),
         m_zoomLevel(8.0),
         m_componentCompleted(false),
         m_mappingManagerInitialized(false),
-        m_pendingFitViewport(false)
+        m_color(QColor::fromRgbF(0.9, 0.9, 0.9)),
+        m_pendingFitViewport(false),
+        m_validRegion(false)
 {
     setAcceptHoverEvents(false);
     setAcceptedMouseButtons(Qt::LeftButton);
@@ -190,8 +180,6 @@ QDeclarativeGeoMap::QDeclarativeGeoMap(QQuickItem *parent)
 
     connect(this, SIGNAL(childrenChanged()), this, SLOT(onMapChildrenChanged()), Qt::QueuedConnection);
 
-    // Create internal flickable and pinch area.
-    m_gestureArea = new QDeclarativeGeoMapGestureArea(this, this);
     m_activeMapType = new QDeclarativeGeoMapType(QGeoMapType(QGeoMapType::NoMap,
                                                              tr("No Map"),
                                                              tr("No Map"), false, false, 0), this);
@@ -261,6 +249,43 @@ void QDeclarativeGeoMap::onMapChildrenChanged()
     copyrights->setCopyrightsZ(maxChildZ + 1);
 }
 
+static QDeclarativeGeoMapType *findMapType(const QList<QDeclarativeGeoMapType *> &types, const QGeoMapType &type)
+{
+    for (int i = 0; i < types.size(); ++i)
+        if (types[i]->mapType() == type)
+            return types[i];
+    return Q_NULLPTR;
+}
+
+void QDeclarativeGeoMap::onSupportedMapTypesChanged()
+{
+    QList<QDeclarativeGeoMapType *> supportedMapTypes;
+    QList<QGeoMapType> types = m_mappingManager->supportedMapTypes();
+    for (int i = 0; i < types.size(); ++i) {
+        // types that are present and get removed will be deleted at QObject destruction
+        QDeclarativeGeoMapType *type = findMapType(m_supportedMapTypes, types[i]);
+        if (!type)
+            type = new QDeclarativeGeoMapType(types[i], this);
+        supportedMapTypes.append(type);
+    }
+    m_supportedMapTypes.swap(supportedMapTypes);
+    if (m_supportedMapTypes.isEmpty()) {
+        m_map->setActiveMapType(QGeoMapType()); // no supported map types: setting an invalid one
+    } else {
+        bool hasMapType = false;
+        foreach (QDeclarativeGeoMapType *declarativeType, m_supportedMapTypes) {
+            if (declarativeType->mapType() == m_map->activeMapType())
+                hasMapType = true;
+        }
+        if (!hasMapType) {
+            QDeclarativeGeoMapType *type = m_supportedMapTypes.at(0);
+            m_activeMapType = type;
+            m_map->setActiveMapType(type->mapType());
+        }
+    }
+
+    emit supportedMapTypesChanged();
+}
 
 void QDeclarativeGeoMap::setError(QGeoServiceProvider::Error error, const QString &errorString)
 {
@@ -279,11 +304,14 @@ void QDeclarativeGeoMap::pluginReady()
     m_serviceProvider = m_plugin->sharedGeoServiceProvider();
     m_mappingManager = m_serviceProvider->mappingManager();
 
-    setError(m_serviceProvider->error(), m_serviceProvider->errorString());
+    if (m_serviceProvider->error() != QGeoServiceProvider::NoError) {
+        setError(m_serviceProvider->error(), m_serviceProvider->errorString());
+        return;
+    }
 
-    if (!m_mappingManager || m_serviceProvider->error() != QGeoServiceProvider::NoError) {
-        qmlInfo(this) << QStringLiteral("Error: Plugin does not support mapping.\nError message:")
-                      << m_serviceProvider->errorString();
+    if (!m_mappingManager) {
+        //TODO Should really be EngineNotSetError (see QML GeoCodeModel)
+        setError(QGeoServiceProvider::NotSupportedError, tr("Plugin does not support mapping."));
         return;
     }
 
@@ -368,7 +396,7 @@ void QDeclarativeGeoMap::touchUngrabEvent()
     further details.
 */
 
-QDeclarativeGeoMapGestureArea *QDeclarativeGeoMap::gesture()
+QQuickGeoMapGestureArea *QDeclarativeGeoMap::gesture()
 {
     return m_gestureArea;
 }
@@ -379,6 +407,10 @@ QDeclarativeGeoMapGestureArea *QDeclarativeGeoMap::gesture()
 void QDeclarativeGeoMap::populateMap()
 {
     QObjectList kids = children();
+    QList<QQuickItem *> quickKids = childItems();
+    for (int i=0; i < quickKids.count(); ++i)
+        kids.append(quickKids.at(i));
+
     for (int i = 0; i < kids.size(); ++i) {
         // dispatch items appropriately
         QDeclarativeGeoMapItemView *mapView = qobject_cast<QDeclarativeGeoMapItemView *>(kids.at(i));
@@ -416,9 +448,11 @@ QSGNode *QDeclarativeGeoMap::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDa
 
     QSGSimpleRectNode *root = static_cast<QSGSimpleRectNode *>(oldNode);
     if (!root)
-        root = new QSGSimpleRectNode(boundingRect(), QColor::fromRgbF(0.9, 0.9, 0.9));
-    else
+        root = new QSGSimpleRectNode(boundingRect(), m_color);
+    else {
         root->setRect(boundingRect());
+        root->setColor(m_color);
+    }
 
     QSGNode *content = root->childCount() ? root->firstChild() : 0;
     content = m_map->updateSceneGraph(content, window());
@@ -428,7 +462,6 @@ QSGNode *QDeclarativeGeoMap::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDa
     return root;
 }
 
-
 /*!
     \qmlproperty Plugin QtLocation::Map::plugin
 
@@ -496,7 +529,7 @@ void QDeclarativeGeoMap::mappingManagerInitialized()
     connect(m_map->mapController(),
             SIGNAL(centerChanged(QGeoCoordinate)),
             this,
-            SIGNAL(centerChanged(QGeoCoordinate)));
+            SLOT(mapCenterChanged(QGeoCoordinate)));
     connect(m_map->mapController(),
             SIGNAL(zoomChanged(qreal)),
             this,
@@ -522,6 +555,7 @@ void QDeclarativeGeoMap::mappingManagerInitialized()
     m_map->prefetchData();
     m_map->update();
 
+    connect(m_mappingManager, SIGNAL(supportedMapTypesChanged()), this, SLOT(onSupportedMapTypesChanged()));
     emit minimumZoomLevelChanged();
     emit maximumZoomLevelChanged();
     emit supportedMapTypesChanged();
@@ -535,16 +569,6 @@ void QDeclarativeGeoMap::mappingManagerInitialized()
     }
 }
 
-/*!
-    \internal
-*/
-void QDeclarativeGeoMap::updateMapDisplay(const QRectF &target)
-{
-    Q_UNUSED(target);
-    QQuickItem::update();
-}
-
-
 /*!
     \internal
 */
@@ -649,6 +673,8 @@ void QDeclarativeGeoMap::setZoomLevel(qreal zoomLevel)
         return;
 
     m_zoomLevel = zoomLevel;
+    m_validRegion = false;
+
     if (m_mappingManagerInitialized)
         m_map->mapController()->setZoom(m_zoomLevel);
     emit zoomLevelChanged(zoomLevel);
@@ -679,6 +705,7 @@ void QDeclarativeGeoMap::setCenter(const QGeoCoordinate &center)
         return;
 
     m_center = center;
+    m_validRegion = false;
 
     if (m_center.isValid() && m_mappingManagerInitialized) {
         m_map->mapController()->setCenter(m_center);
@@ -713,11 +740,11 @@ QGeoCoordinate QDeclarativeGeoMap::center() const
 
     This property does not provide any change notifications.
 
-    \since 5.5
+    \since 5.6
 */
 void QDeclarativeGeoMap::setVisibleRegion(const QGeoShape &shape)
 {
-    if (shape == m_region)
+    if (shape == m_region && m_validRegion)
         return;
 
     m_region = shape;
@@ -737,8 +764,8 @@ void QDeclarativeGeoMap::setVisibleRegion(const QGeoShape &shape)
 
 QGeoShape QDeclarativeGeoMap::visibleRegion() const
 {
-    if (!width() || !height())
-        return QGeoShape();
+    if (!m_map || !width() || !height())
+        return m_region;
 
     QGeoCoordinate tl = m_map->itemPositionToCoordinate(QDoubleVector2D(0, 0));
     QGeoCoordinate br = m_map->itemPositionToCoordinate(QDoubleVector2D(width(), height()));
@@ -746,32 +773,90 @@ QGeoShape QDeclarativeGeoMap::visibleRegion() const
     return QGeoRectangle(tl, br);
 }
 
+
+/*!
+    \qmlproperty color QtLocation::Map::color
+
+    This property holds the background color of the map element.
+
+    \since 5.6
+*/
+void QDeclarativeGeoMap::setColor(const QColor &color)
+{
+    if (color != m_color) {
+        m_color = color;
+        update();
+        emit colorChanged(m_color);
+    }
+}
+
+QColor QDeclarativeGeoMap::color() const
+{
+    return m_color;
+}
+
 void QDeclarativeGeoMap::fitViewportToGeoShape()
 {
-    double bboxWidth;
-    double bboxHeight;
-    QGeoCoordinate centerCoordinate;
+    int margins  = 10;
+    if (!m_map || width() <= margins || height() <= margins)
+        return;
+
+    QGeoCoordinate topLeft;
+    QGeoCoordinate bottomRight;
 
     switch (m_region.type()) {
     case QGeoShape::RectangleType:
     {
         QGeoRectangle rect = m_region;
-        QDoubleVector2D topLeftPoint = m_map->coordinateToItemPosition(rect.topLeft(), false);
-        QDoubleVector2D botRightPoint = m_map->coordinateToItemPosition(rect.bottomRight(), false);
-        bboxWidth = qAbs(topLeftPoint.x() - botRightPoint.x());
-        bboxHeight = qAbs(topLeftPoint.y() - botRightPoint.y());
-        centerCoordinate = rect.center();
+        topLeft = rect.topLeft();
+        bottomRight = rect.bottomRight();
+        if (bottomRight.longitude() < topLeft.longitude())
+            bottomRight.setLongitude(bottomRight.longitude() + 360.0);
+
         break;
     }
     case QGeoShape::CircleType:
     {
+        const double pi = M_PI;
         QGeoCircle circle = m_region;
-        centerCoordinate = circle.center();
-        QGeoCoordinate edge = centerCoordinate.atDistanceAndAzimuth(circle.radius(), 90);
-        QDoubleVector2D centerPoint = m_map->coordinateToItemPosition(centerCoordinate, false);
-        QDoubleVector2D edgePoint = m_map->coordinateToItemPosition(edge, false);
-        bboxWidth = qAbs(centerPoint.x() - edgePoint.x()) * 2;
-        bboxHeight = bboxWidth;
+        QGeoCoordinate centerCoordinate = circle.center();
+
+        // calculate geo bounding box of the circle
+        // circle tangential points with meridians and the north pole create
+        // spherical triangle, we use spherical law of sines
+        // sin(lon_delta_in_rad)/sin(r_in_rad) =
+        // sin(alpha_in_rad)/sin(pi/2 - lat_in_rad), where:
+        // * lon_delta_in_rad - delta of longitudes of circle center
+        //   and tangential points
+        // * r_in_rad - angular radius of the circle
+        // * lat_in_rad - latitude of circle center
+        // * alpha_in_rad - angle between meridian and radius to the circle =>
+        //   this is tangential point => sin(alpha) = 1
+        // * lat_delta_in_rad - delta of latitudes of circle center and
+        //   latitude of points where great circle (going through circle
+        //   center) crosses circle and the pole
+
+        double r_in_rad = circle.radius() / EARTH_MEAN_RADIUS; // angular r
+        double lat_delta_in_deg = r_in_rad * 180 / pi;
+        double lon_delta_in_deg = std::asin(std::sin(r_in_rad) /
+               std::cos(centerCoordinate.latitude() * pi / 180)) * 180 / pi;
+
+        topLeft.setLatitude(centerCoordinate.latitude() + lat_delta_in_deg);
+        topLeft.setLongitude(centerCoordinate.longitude() - lon_delta_in_deg);
+        bottomRight.setLatitude(centerCoordinate.latitude()
+                                - lat_delta_in_deg);
+        bottomRight.setLongitude(centerCoordinate.longitude()
+                                 + lon_delta_in_deg);
+
+        // adjust if circle reaches poles => cross all meridians and
+        // fit into Mercator projection bounds
+        if (topLeft.latitude() > 90 || bottomRight.latitude() < -90) {
+            topLeft.setLatitude(qMin(topLeft.latitude(), 85.05113));
+            topLeft.setLongitude(-180.0);
+            bottomRight.setLatitude(qMax(bottomRight.latitude(),
+                                                 -85.05113));
+            bottomRight.setLongitude(180.0);
+        }
         break;
     }
     case QGeoShape::UnknownType:
@@ -780,27 +865,37 @@ void QDeclarativeGeoMap::fitViewportToGeoShape()
         return;
     }
 
-    // position camera to the center of bounding box
-    setProperty("center", QVariant::fromValue(centerCoordinate));
+    // adjust zoom, use reference world to keep things simple
+    // otherwise we would need to do the error prone longitudes
+    // wrapping
+    QDoubleVector2D topLeftPoint =
+            m_map->referenceCoordinateToItemPosition(topLeft);
+    QDoubleVector2D bottomRightPoint =
+            m_map->referenceCoordinateToItemPosition(bottomRight);
 
-    //If the shape is empty we just change centerposition, not zoom
+    double bboxWidth = bottomRightPoint.x() - topLeftPoint.x();
+    double bboxHeight = bottomRightPoint.y() - topLeftPoint.y();
+
+    // find center of the bounding box
+    QGeoCoordinate centerCoordinate =
+            m_map->referenceItemPositionToCoordinate(
+                (topLeftPoint + bottomRightPoint)/2);
+
+    // position camera to the center of bounding box
+    setCenter(centerCoordinate);
+
+    // if the shape is empty we just change center position, not zoom
     if (bboxHeight == 0 && bboxWidth == 0)
         return;
 
-    // adjust zoom
-    double bboxWidthRatio = bboxWidth / (bboxWidth + bboxHeight);
-    double mapWidthRatio = width() / (width() + height());
-    double zoomRatio;
-
-    if (bboxWidthRatio > mapWidthRatio)
-        zoomRatio = bboxWidth / width();
-    else
-        zoomRatio = bboxHeight / height();
-
-    qreal newZoom = std::log10(zoomRatio) / std::log10(0.5);
-
-    newZoom = std::floor(qMax(minimumZoomLevel(), (m_map->mapController()->zoom() + newZoom)));
-    setProperty("zoomLevel", QVariant::fromValue(newZoom));
+    double zoomRatio = qMax(bboxWidth / (width() - margins),
+                            bboxHeight / (height() - margins));
+    // fixme: use log2 with c++11
+    zoomRatio = std::log(zoomRatio) / std::log(2.0);
+    double newZoom = qMax<double>(minimumZoomLevel(), m_map->mapController()->zoom()
+                          - zoomRatio);
+    setZoomLevel(newZoom);
+    m_validRegion = true;
 }
 
 /*!
@@ -811,9 +906,16 @@ void QDeclarativeGeoMap::mapZoomLevelChanged(qreal zoom)
     if (zoom == m_zoomLevel)
         return;
     m_zoomLevel = zoom;
+    m_validRegion = false;
     emit zoomLevelChanged(m_zoomLevel);
 }
 
+void QDeclarativeGeoMap::mapCenterChanged(const QGeoCoordinate &center)
+{
+    m_validRegion = false;
+    emit centerChanged(center);
+}
+
 /*!
     \qmlproperty list<MapType> QtLocation::Map::supportedMapTypes
 
@@ -827,47 +929,37 @@ QQmlListProperty<QDeclarativeGeoMapType> QDeclarativeGeoMap::supportedMapTypes()
 }
 
 /*!
-    \qmlmethod coordinate QtLocation::Map::toCoordinate(QPointF position)
+    \qmlmethod coordinate QtLocation::Map::toCoordinate(QPointF position, bool clipToViewPort)
 
     Returns the coordinate which corresponds to the \a position relative to the map item.
 
-    Returns an invalid coordinate if \a position is not within the current viewport.
+    If \a cliptoViewPort is \c true, or not supplied then returns an invalid coordinate if
+    \a position is not within the current viewport.
 */
-QGeoCoordinate QDeclarativeGeoMap::toCoordinate(const QPointF &position) const
+QGeoCoordinate QDeclarativeGeoMap::toCoordinate(const QPointF &position, bool clipToViewPort) const
 {
     if (m_map)
-        return m_map->itemPositionToCoordinate(QDoubleVector2D(position));
+        return m_map->itemPositionToCoordinate(QDoubleVector2D(position), clipToViewPort);
     else
         return QGeoCoordinate();
 }
 
 /*!
-    \qmlmethod point QtLocation::Map::fromCoordinate(coordinate coordinate)
+    \qmlmethod point QtLocation::Map::fromCoordinate(coordinate coordinate, bool clipToViewPort)
 
     Returns the position relative to the map item which corresponds to the \a coordinate.
 
-    Returns an invalid QPointF if \a coordinate is not within the current viewport.
+    If \a cliptoViewPort is \c true, or not supplied then returns an invalid QPointF if
+    \a coordinate is not within the current viewport.
 */
-QPointF QDeclarativeGeoMap::fromCoordinate(const QGeoCoordinate &coordinate) const
+QPointF QDeclarativeGeoMap::fromCoordinate(const QGeoCoordinate &coordinate, bool clipToViewPort) const
 {
     if (m_map)
-        return m_map->coordinateToItemPosition(coordinate).toPointF();
+        return m_map->coordinateToItemPosition(coordinate, clipToViewPort).toPointF();
     else
         return QPointF(qQNaN(), qQNaN());
 }
 
-/*!
-    \qmlmethod QtLocation::Map::toScreenPosition(coordinate coordinate)
-    \obsolete
-
-    This function is missed named and is equilavent to \l {fromCoordinate}, which should be used
-    instead.
-*/
-QPointF QDeclarativeGeoMap::toScreenPosition(const QGeoCoordinate &coordinate) const
-{
-    return fromCoordinate(coordinate);
-}
-
 /*!
     \qmlmethod void QtLocation::Map::pan(int dx, int dy)
 
@@ -899,6 +991,18 @@ void QDeclarativeGeoMap::prefetchData()
     m_map->prefetchData();
 }
 
+/*!
+    \qmlmethod void QtLocation::Map::clearData()
+
+    Clears map data collected by the currently selected plugin.
+    \note This method will delete cached files.
+    \sa plugin
+*/
+void QDeclarativeGeoMap::clearData()
+{
+    m_map->clearData();
+}
+
 /*!
     \qmlproperty string QtLocation::Map::errorString
 
@@ -923,7 +1027,7 @@ QString QDeclarativeGeoMap::errorString() const
 
     \list
     \li Map.NoError - No error has occurred.
-    \li Map.NotSupportedError -The plugin does not support mapping functionality.
+    \li Map.NotSupportedError -The maps plugin property was not set or there is no mapping manager associated with the plugin.
     \li Map.UnknownParameterError -The plugin did not recognize one of the parameters it was given.
     \li Map.MissingRequiredParameterError - The plugin did not find one of the parameters it was expecting.
     \li Map.ConnectionError - The plugin could not connect to its backend service or database.
@@ -948,9 +1052,10 @@ void QDeclarativeGeoMap::touchEvent(QTouchEvent *event)
              event->type() == QEvent::TouchCancel) {
             ungrabTouchPoints();
         }
+    } else {
+        //ignore event so sythesized event is generated;
+        QQuickItem::touchEvent(event);
     }
-    //this will always ignore event so sythesized event is generated;
-    QQuickItem::touchEvent(event);
 }
 
 /*!
@@ -967,7 +1072,7 @@ void QDeclarativeGeoMap::wheelEvent(QWheelEvent *event)
 
 bool QDeclarativeGeoMap::isInteractive()
 {
-    return (m_gestureArea->enabled() && m_gestureArea->activeGestures()) || m_gestureArea->isActive();
+    return (m_gestureArea->enabled() && m_gestureArea->acceptedGestures()) || m_gestureArea->isActive();
 }
 
 /*!
@@ -1144,6 +1249,8 @@ QDeclarativeGeoMapType * QDeclarativeGeoMap::activeMapType() const
 */
 void QDeclarativeGeoMap::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
+    m_gestureArea->setSize(newGeometry.size());
+
     if (!m_mappingManagerInitialized)
         return;
 
@@ -1165,89 +1272,6 @@ void QDeclarativeGeoMap::geometryChanged(const QRectF &newGeometry, const QRectF
 
 }
 
-// TODO Remove this function -> BC break
-/*!
-    \qmlmethod void QtLocation::Map::fitViewportToGeoShape(QGeoShape shape)
-
-    \internal
-
-    Fits the current viewport to the boundary of the shape. The camera is positioned
-    in the center of the shape, and at the largest integral zoom level possible which
-    allows the whole shape to be visible on screen
-
-*/
-void QDeclarativeGeoMap::fitViewportToGeoShape(const QVariant &variantShape)
-{
-    if (!m_map || !m_mappingManagerInitialized)
-        return;
-
-    QGeoShape shape;
-
-    if (variantShape.userType() == qMetaTypeId<QGeoRectangle>())
-        shape = variantShape.value<QGeoRectangle>();
-    else if (variantShape.userType() == qMetaTypeId<QGeoCircle>())
-        shape = variantShape.value<QGeoCircle>();
-    else if (variantShape.userType() == qMetaTypeId<QGeoShape>())
-        shape = variantShape.value<QGeoShape>();
-
-    if (!shape.isValid())
-        return;
-
-    double bboxWidth;
-    double bboxHeight;
-    QGeoCoordinate centerCoordinate;
-
-    switch (shape.type()) {
-    case QGeoShape::RectangleType:
-    {
-        QGeoRectangle rect = shape;
-        QDoubleVector2D topLeftPoint = m_map->coordinateToItemPosition(rect.topLeft(), false);
-        QDoubleVector2D botRightPoint = m_map->coordinateToItemPosition(rect.bottomRight(), false);
-        bboxWidth = qAbs(topLeftPoint.x() - botRightPoint.x());
-        bboxHeight = qAbs(topLeftPoint.y() - botRightPoint.y());
-        centerCoordinate = rect.center();
-        break;
-    }
-    case QGeoShape::CircleType:
-    {
-        QGeoCircle circle = shape;
-        centerCoordinate = circle.center();
-        QGeoCoordinate edge = centerCoordinate.atDistanceAndAzimuth(circle.radius(), 90);
-        QDoubleVector2D centerPoint = m_map->coordinateToItemPosition(centerCoordinate, false);
-        QDoubleVector2D edgePoint = m_map->coordinateToItemPosition(edge, false);
-        bboxWidth = qAbs(centerPoint.x() - edgePoint.x()) * 2;
-        bboxHeight = bboxWidth;
-        break;
-    }
-    case QGeoShape::UnknownType:
-        //Fallthrough to default
-    default:
-        return;
-    }
-
-    // position camera to the center of bounding box
-    setProperty("center", QVariant::fromValue(centerCoordinate));
-
-    //If the shape is empty we just change centerposition, not zoom
-    if (bboxHeight == 0 && bboxWidth == 0)
-        return;
-
-    // adjust zoom
-    double bboxWidthRatio = bboxWidth / (bboxWidth + bboxHeight);
-    double mapWidthRatio = width() / (width() + height());
-    double zoomRatio;
-
-    if (bboxWidthRatio > mapWidthRatio)
-        zoomRatio = bboxWidth / width();
-    else
-        zoomRatio = bboxHeight / height();
-
-    qreal newZoom = std::log10(zoomRatio) / std::log10(0.5);
-
-    newZoom = std::floor(qMax(minimumZoomLevel(), (m_map->mapController()->zoom() + newZoom)));
-    setProperty("zoomLevel", QVariant::fromValue(newZoom));
-}
-
 /*!
     \qmlmethod void QtLocation::Map::fitViewportToMapItems()
 
@@ -1299,6 +1323,10 @@ void QDeclarativeGeoMap::fitViewportToMapItemsRefine(bool refine)
                 continue;
             }
         }
+        // Force map items to update immediately. Needed to ensure correct item size and positions
+        // when recursively calling this function.
+        if (item->isPolishScheduled())
+           item->updatePolish();
 
         topLeftX = item->position().x();
         topLeftY = item->position().y();
