@@ -1,39 +1,34 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtLocation module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL3$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or later as published by the Free
+** Software Foundation and appearing in the file LICENSE.GPL included in
+** the packaging of this file. Please review the following information to
+** ensure the GNU General Public License version 2.0 requirements will be
+** met: http://www.gnu.org/licenses/gpl-2.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -57,7 +52,7 @@ QT_BEGIN_NAMESPACE
     \instantiates QDeclarativeGeoMapQuickItem
     \inqmlmodule QtLocation
     \ingroup qml-QtLocation5-maps
-    \since Qt Location 5.0
+    \since Qt Location 5.5
 
     \brief The MapQuickItem type displays an arbitrary Qt Quick object
            on a Map.
@@ -111,16 +106,16 @@ QT_BEGIN_NAMESPACE
     to display a Marker on the Map. This strategy is used to show the map
     markers in the MapViewer example.
 
-    \snippet mapviewer/content/map/Marker.qml mqi-top
-    \snippet mapviewer/content/map/Marker.qml mqi-anchor
-    \snippet mapviewer/content/map/Marker.qml mqi-closeimage
-    \snippet mapviewer/content/map/Marker.qml mqi-close
+    \snippet mapviewer/map/Marker.qml mqi-top
+    \snippet mapviewer/map/Marker.qml mqi-anchor
+    \snippet mapviewer/map/Marker.qml mqi-closeimage
+    \snippet mapviewer/map/Marker.qml mqi-close
 
     \image api-mapquickitem.png
 */
 
 QDeclarativeGeoMapQuickItem::QDeclarativeGeoMapQuickItem(QQuickItem *parent)
-:   QDeclarativeGeoMapItemBase(parent), zoomLevel_(0.0), inUpdate_(false),
+:   QDeclarativeGeoMapItemBase(parent), zoomLevel_(0.0),
     mapAndSourceItemSet_(false), updatingGeometry_(false)
 {
     setFlag(ItemHasContents, true);
@@ -151,8 +146,7 @@ void QDeclarativeGeoMapQuickItem::setCoordinate(const QGeoCoordinate &coordinate
 
     coordinate_ = coordinate;
 
-    updateMapItem();
-
+    polishAndUpdate();
     emit coordinateChanged();
 }
 
@@ -163,10 +157,9 @@ void QDeclarativeGeoMapQuickItem::setMap(QDeclarativeGeoMap *quickMap, QGeoMap *
 {
     QDeclarativeGeoMapItemBase::setMap(quickMap,map);
     if (map && quickMap) {
-        QObject::connect(quickMap, SIGNAL(heightChanged()), this, SLOT(updateMapItem()));
-        QObject::connect(quickMap, SIGNAL(widthChanged()), this, SLOT(updateMapItem()));
-        QObject::connect(map, SIGNAL(cameraDataChanged(QGeoCameraData)), this, SLOT(updateMapItem()));
-        updateMapItem();
+        connect(map, SIGNAL(cameraDataChanged(QGeoCameraData)),
+                this, SLOT(polishAndUpdate()));
+        polishAndUpdate();
     }
 }
 
@@ -181,7 +174,7 @@ void QDeclarativeGeoMapQuickItem::geometryChanged(const QRectF &newGeometry, con
         return;
     }
 
-    QGeoCoordinate newCoordinate = map()->screenPositionToCoordinate(QDoubleVector2D(x(), y()) + (scaleFactor() * QDoubleVector2D(anchorPoint_)), false);
+    QGeoCoordinate newCoordinate = map()->itemPositionToCoordinate(QDoubleVector2D(x(), y()) + (scaleFactor() * QDoubleVector2D(anchorPoint_)), false);
     if (newCoordinate.isValid())
         setCoordinate(newCoordinate);
 
@@ -208,8 +201,7 @@ void QDeclarativeGeoMapQuickItem::setSourceItem(QQuickItem *sourceItem)
         return;
     sourceItem_ = sourceItem;
 
-    updateMapItem();
-
+    polishAndUpdate();
     emit sourceItemChanged();
 }
 
@@ -254,7 +246,7 @@ void QDeclarativeGeoMapQuickItem::setAnchorPoint(const QPointF &anchorPoint)
     if (anchorPoint == anchorPoint_)
         return;
     anchorPoint_ = anchorPoint;
-    updateMapItem();
+    polishAndUpdate();
     emit anchorPointChanged();
 }
 
@@ -286,7 +278,7 @@ void QDeclarativeGeoMapQuickItem::setZoomLevel(qreal zoomLevel)
     if (zoomLevel == zoomLevel_)
         return;
     zoomLevel_ = zoomLevel;
-    updateMapItem();
+    polishAndUpdate();
     emit zoomLevelChanged();
 }
 
@@ -298,7 +290,7 @@ qreal QDeclarativeGeoMapQuickItem::zoomLevel() const
 /*!
     \internal
 */
-void QDeclarativeGeoMapQuickItem::updateMapItem()
+void QDeclarativeGeoMapQuickItem::updatePolish()
 {
     if (!quickMap() && sourceItem_) {
         mapAndSourceItemSet_ = false;
@@ -316,13 +308,13 @@ void QDeclarativeGeoMapQuickItem::updateMapItem()
         sourceItem_.data()->setParentItem(opacityContainer_);
         sourceItem_.data()->setTransformOrigin(QQuickItem::TopLeft);
         connect(sourceItem_.data(), SIGNAL(xChanged()),
-                this, SLOT(updateMapItem()));
+                this, SLOT(polishAndUpdate()));
         connect(sourceItem_.data(), SIGNAL(yChanged()),
-                this, SLOT(updateMapItem()));
+                this, SLOT(polishAndUpdate()));
         connect(sourceItem_.data(), SIGNAL(widthChanged()),
-                this, SLOT(updateMapItem()));
+                this, SLOT(polishAndUpdate()));
         connect(sourceItem_.data(), SIGNAL(heightChanged()),
-                this, SLOT(updateMapItem()));
+                this, SLOT(polishAndUpdate()));
     }
 
     QScopedValueRollback<bool> rollback(updatingGeometry_);
@@ -335,7 +327,6 @@ void QDeclarativeGeoMapQuickItem::updateMapItem()
     setWidth(sourceItem_.data()->width());
     setHeight(sourceItem_.data()->height());
     setPositionOnMap(coordinate(), scaleFactor() * anchorPoint_);
-    update();
 }
 
 /*!
