@@ -1,57 +1,45 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtLocation module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL3$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or later as published by the Free
+** Software Foundation and appearing in the file LICENSE.GPL included in
+** the packaging of this file. Please review the following information to
+** ensure the GNU General Public License version 2.0 requirements will be
+** met: http://www.gnu.org/licenses/gpl-2.0.html.
 **
 ** $QT_END_LICENSE$
 **
-** This file is part of the Nokia services plugin for the Maps and
-** Navigation API.  The use of these services, whether by use of the
-** plugin or by other means, is governed by the terms and conditions
-** described by the file NOKIA_TERMS_AND_CONDITIONS.txt in
-** this package, located in the directory containing the Nokia services
-** plugin source code.
-**
 ****************************************************************************/
 
 #include "qplacemanagerengine_nokiav2.h"
 
-#include "placesv2/qplacecategoriesreplyimpl.h"
+#include "placesv2/qplacecategoriesreplyhere.h"
 #include "placesv2/qplacecontentreplyimpl.h"
 #include "placesv2/qplacesearchsuggestionreplyimpl.h"
-#include "placesv2/qplacesearchreplyimpl.h"
+#include "placesv2/qplacesearchreplyhere.h"
 #include "placesv2/qplacedetailsreplyimpl.h"
 #include "placesv2/qplaceidreplyimpl.h"
 #include "qgeonetworkaccessmanager.h"
@@ -84,11 +72,20 @@ static const char FIXED_CATEGORIES_string[] =
     "leisure-outdoor\0"
     "administrative-areas-buildings\0"
     "natural-geographical\0"
+    "petrol-station\0"
+    "atm-bank-exchange\0"
+    "toilet-rest-area\0"
+    "hospital-health-care-facility\0"
+    "eat-drink|restaurant\0" // subcategories always start after relative parent category
+    "eat-drink|coffee-tea\0"
+    "eat-drink|snacks-fast-food\0"
+    "transport|airport"
     "\0";
 
 static const int FIXED_CATEGORIES_indices[] = {
        0,   10,   20,   35,   45,   59,   68,   84,
-     115,   -1
+     115,  136,  151,  169,  186,  216,  237,  258,
+     285,   -1
 };
 
 static const char * const NokiaIcon = "nokiaIcon";
@@ -136,20 +133,20 @@ bool CategoryParser::parse(const QString &fileName)
         QJsonDocument document = QJsonDocument::fromJson(mappingFile.readAll());
         if (document.isObject()) {
             QJsonObject docObject = document.object();
-            if (docObject.contains(QLatin1String("offline_explore"))) {
-                m_exploreObject = docObject.value(QLatin1String("offline_explore"))
+            if (docObject.contains(QStringLiteral("offline_explore"))) {
+                m_exploreObject = docObject.value(QStringLiteral("offline_explore"))
                                                 .toObject();
-                if (m_exploreObject.contains(QLatin1String("ROOT"))) {
+                if (m_exploreObject.contains(QStringLiteral("ROOT"))) {
                     processCategory(0, QString());
                     return true;
                 }
             } else {
-                m_errorString = fileName + QLatin1String("does not contain the "
-                                                       "offline_explore property");
+                m_errorString = fileName
+                    + QStringLiteral("does not contain the offline_explore property");
                 return false;
             }
         } else {
-            m_errorString = fileName + QLatin1String("is not an json object");
+            m_errorString = fileName + QStringLiteral("is not an json object");
             return false;
         }
     }
@@ -180,10 +177,10 @@ void CategoryParser::processCategory(int level, const QString &id, const QString
     //a proper assignment to the tree happens at the end of function
 
     QJsonObject categoryJson = m_exploreObject.value(id.isEmpty()
-                                                     ? QLatin1String("ROOT") : id).toObject();
-    QJsonArray children = categoryJson.value(QLatin1String("children")).toArray();
+                                                     ? QStringLiteral("ROOT") : id).toObject();
+    QJsonArray children = categoryJson.value(QStringLiteral("children")).toArray();
 
-    if (level + 1 <= maxLevel && !categoryJson.contains(QLatin1String("final"))) {
+    if (level + 1 <= maxLevel && !categoryJson.contains(QStringLiteral("final"))) {
         for (int i = 0; i < children.count(); ++i)  {
             QString childId = children.at(i).toString();
             if (!m_tree.contains(childId)) {
@@ -198,24 +195,24 @@ void CategoryParser::processCategory(int level, const QString &id, const QString
 
 QPlaceManagerEngineNokiaV2::QPlaceManagerEngineNokiaV2(
     QGeoNetworkAccessManager *networkManager,
-    const QMap<QString, QVariant> &parameters,
+    const QVariantMap &parameters,
     QGeoServiceProvider::Error *error,
     QString *errorString)
     : QPlaceManagerEngine(parameters)
     , m_manager(networkManager)
-    , m_uriProvider(new QGeoUriProvider(this, parameters, "places.host", PLACES_HOST, PLACES_HOST_CN))
+    , m_uriProvider(new QGeoUriProvider(this, parameters, QStringLiteral("here.places.host"), PLACES_HOST))
 {
     Q_ASSERT(networkManager);
     m_manager->setParent(this);
 
     m_locales.append(QLocale());
 
-    m_appId = parameters.value(QLatin1String("app_id")).toString();
-    m_appCode = parameters.value(QLatin1String("token")).toString();
+    m_appId = parameters.value(QStringLiteral("here.app_id")).toString();
+    m_appCode = parameters.value(QStringLiteral("here.token")).toString();
 
     m_theme = parameters.value(IconThemeKey, QString()).toString();
 
-    if (m_theme == QLatin1String("default"))
+    if (m_theme == QStringLiteral("default"))
         m_theme.clear();
 
     m_localDataPath = parameters.value(LocalDataPathKey, QString()).toString();
@@ -224,7 +221,7 @@ QPlaceManagerEngineNokiaV2::QPlaceManagerEngineNokiaV2(
 
         if (!dataLocations.isEmpty() && !dataLocations.first().isEmpty()) {
             m_localDataPath = dataLocations.first()
-                                + QLatin1String("/nokia/qtlocation/data");
+                                + QStringLiteral("/here/qtlocation/data");
         }
     }
 
@@ -240,13 +237,13 @@ QPlaceManagerEngineNokiaV2::~QPlaceManagerEngineNokiaV2() {}
 QPlaceDetailsReply *QPlaceManagerEngineNokiaV2::getPlaceDetails(const QString &placeId)
 {
     QUrl requestUrl(QString::fromLatin1("http://") + m_uriProvider->getCurrentHost() +
-                    QLatin1String("/places/v1/places/") + placeId);
+                    QStringLiteral("/places/v1/places/") + placeId);
 
     QUrlQuery queryItems;
 
-    queryItems.addQueryItem(QLatin1String("tf"), QLatin1String("html"));
-    //queryItems.append(qMakePair<QString, QString>(QLatin1String("size"), QString::number(5)));
-    //queryItems.append(qMakePair<QString, QString>(QLatin1String("image_dimensions"), QLatin1String("w64-h64,w100")));
+    queryItems.addQueryItem(QStringLiteral("tf"), QStringLiteral("html"));
+    //queryItems.append(qMakePair<QString, QString>(QStringLiteral("size"), QString::number(5)));
+    //queryItems.append(qMakePair<QString, QString>(QStringLiteral("image_dimensions"), QStringLiteral("w64-h64,w100")));
 
     requestUrl.setQuery(queryItems);
 
@@ -261,75 +258,63 @@ QPlaceDetailsReply *QPlaceManagerEngineNokiaV2::getPlaceDetails(const QString &p
     return reply;
 }
 
-QPlaceContentReply *QPlaceManagerEngineNokiaV2::getPlaceContent(const QString &placeId,
-                                                              const QPlaceContentRequest &request)
+QPlaceContentReply *QPlaceManagerEngineNokiaV2::getPlaceContent(const QPlaceContentRequest &request)
 {
-    QUrl requestUrl(QString::fromLatin1("http://") + m_uriProvider->getCurrentHost() +
-                    QLatin1String("/places/v1/places/") + placeId + QLatin1String("/media/"));
-
     QNetworkReply *networkReply = 0;
 
-    QUrlQuery queryItems;
+    if (request.contentContext().userType() == qMetaTypeId<QUrl>()) {
+        QUrl u = request.contentContext().value<QUrl>();
 
-    switch (request.contentType()) {
-    case QPlaceContent::ImageType:
-        requestUrl.setPath(requestUrl.path() + QLatin1String("images"));
+       networkReply = sendRequest(u);
+    } else {
+        QUrl requestUrl(QString::fromLatin1("http://") + m_uriProvider->getCurrentHost() +
+                        QStringLiteral("/places/v1/places/") + request.placeId() +
+                        QStringLiteral("/media/"));
 
-        queryItems.addQueryItem(QLatin1String("tf"), QLatin1String("html"));
+        QUrlQuery queryItems;
 
-        if (request.limit() > 0) {
-            queryItems.addQueryItem(QLatin1String("size"),
-                                    QString::number(request.limit()));
+        switch (request.contentType()) {
+        case QPlaceContent::ImageType:
+            requestUrl.setPath(requestUrl.path() + QStringLiteral("images"));
+
+            queryItems.addQueryItem(QStringLiteral("tf"), QStringLiteral("html"));
+
+            if (request.limit() > 0)
+                queryItems.addQueryItem(QStringLiteral("size"), QString::number(request.limit()));
+
+            //queryItems.append(qMakePair<QString, QString>(QStringLiteral("image_dimensions"), QStringLiteral("w64-h64,w100")));
+
+            requestUrl.setQuery(queryItems);
+
+            networkReply = sendRequest(requestUrl);
+            break;
+        case QPlaceContent::ReviewType:
+            requestUrl.setPath(requestUrl.path() + QStringLiteral("reviews"));
+
+            queryItems.addQueryItem(QStringLiteral("tf"), QStringLiteral("html"));
+
+            if (request.limit() > 0)
+                queryItems.addQueryItem(QStringLiteral("size"), QString::number(request.limit()));
+
+            requestUrl.setQuery(queryItems);
+
+            networkReply = sendRequest(requestUrl);
+            break;
+        case QPlaceContent::EditorialType:
+            requestUrl.setPath(requestUrl.path() + QStringLiteral("editorials"));
+
+            queryItems.addQueryItem(QStringLiteral("tf"), QStringLiteral("html"));
+
+            if (request.limit() > 0)
+                queryItems.addQueryItem(QStringLiteral("size"), QString::number(request.limit()));
+
+            requestUrl.setQuery(queryItems);
+
+            networkReply = sendRequest(requestUrl);
+            break;
+        case QPlaceContent::NoType:
+            ;
         }
-        if (request.offset() > -1) {
-            queryItems.addQueryItem(QLatin1String("offset"),
-                                    QString::number(request.offset()));
-        }
-
-        //queryItems.append(qMakePair<QString, QString>(QLatin1String("image_dimensions"), QLatin1String("w64-h64,w100")));
-
-        requestUrl.setQuery(queryItems);
-
-        networkReply = sendRequest(requestUrl);
-        break;
-    case QPlaceContent::ReviewType:
-        requestUrl.setPath(requestUrl.path() + QLatin1String("reviews"));
-
-        queryItems.addQueryItem(QLatin1String("tf"), QLatin1String("html"));
-
-        if (request.limit() > 0) {
-            queryItems.addQueryItem(QLatin1String("size"),
-                                    QString::number(request.limit()));
-        }
-        if (request.offset() > -1) {
-            queryItems.addQueryItem(QLatin1String("offset"),
-                                    QString::number(request.offset()));
-        }
-
-        requestUrl.setQuery(queryItems);
-
-        networkReply = sendRequest(requestUrl);
-        break;
-    case QPlaceContent::EditorialType:
-        requestUrl.setPath(requestUrl.path() + QLatin1String("editorials"));
-
-        queryItems.addQueryItem(QLatin1String("tf"), QLatin1String("html"));
-
-        if (request.limit() > 0) {
-            queryItems.addQueryItem(QLatin1String("size"),
-                                    QString::number(request.limit()));
-        }
-        if (request.offset() > -1) {
-            queryItems.addQueryItem(QLatin1String("offset"),
-                                    QString::number(request.offset()));
-        }
-
-        requestUrl.setQuery(queryItems);
-
-        networkReply = sendRequest(requestUrl);
-        break;
-    case QPlaceContent::NoType:
-        ;
     }
 
     QPlaceContentReply *reply = new QPlaceContentReplyImpl(request, networkReply, this);
@@ -349,27 +334,15 @@ QPlaceContentReply *QPlaceManagerEngineNokiaV2::getPlaceContent(const QString &p
 static bool addAtForBoundingArea(const QGeoShape &area,
                                  QUrlQuery *queryItems)
 {
-    QGeoCoordinate center;
-    switch (area.type()) {
-    case QGeoShape::RectangleType:
-        center = QGeoRectangle(area).center();
-        break;
-    case QGeoShape::CircleType:
-        center = QGeoCircle(area).center();
-        break;
-    case QGeoShape::UnknownType:
-        break;
-    }
-
-    if (!center.isValid()) {
+    QGeoCoordinate center = area.center();
+    if (!center.isValid())
         return false;
-    } else {
-        queryItems->addQueryItem(QLatin1String("at"),
-                                 QString::number(center.latitude()) +
-                                 QLatin1Char(',') +
-                                 QString::number(center.longitude()));
-        return true;
-    }
+
+    queryItems->addQueryItem(QStringLiteral("at"),
+                             QString::number(center.latitude()) +
+                             QLatin1Char(',') +
+                             QString::number(center.longitude()));
+    return true;
 }
 
 QPlaceSearchReply *QPlaceManagerEngineNokiaV2::search(const QPlaceSearchRequest &query)
@@ -379,8 +352,6 @@ QPlaceSearchReply *QPlaceManagerEngineNokiaV2::search(const QPlaceSearchRequest
     unsupported |= query.visibilityScope() != QLocation::UnspecifiedVisibility &&
                    query.visibilityScope() != QLocation::PublicVisibility;
 
-    unsupported |= !query.searchTerm().isEmpty() && query.offset() > 0;
-
     // Both a search term and search categories are not supported.
     unsupported |= !query.searchTerm().isEmpty() && !query.categories().isEmpty();
 
@@ -390,7 +361,7 @@ QPlaceSearchReply *QPlaceManagerEngineNokiaV2::search(const QPlaceSearchRequest
                        || query.searchArea().type() != QGeoShape::UnknownType);
 
     if (unsupported) {
-        QPlaceSearchReplyImpl *reply = new QPlaceSearchReplyImpl(query, 0, this);
+        QPlaceSearchReplyHere *reply = new QPlaceSearchReplyHere(query, 0, this);
         connect(reply, SIGNAL(finished()), this, SLOT(replyFinished()));
         connect(reply, SIGNAL(error(QPlaceReply::Error,QString)),
                 this, SLOT(replyError(QPlaceReply::Error,QString)));
@@ -406,7 +377,7 @@ QPlaceSearchReply *QPlaceManagerEngineNokiaV2::search(const QPlaceSearchRequest
     // searches, which do not need search centers.
     if (query.recommendationId().isEmpty() && !query.searchContext().isValid()) {
         if (!addAtForBoundingArea(query.searchArea(), &queryItems)) {
-            QPlaceSearchReplyImpl *reply = new QPlaceSearchReplyImpl(query, 0, this);
+            QPlaceSearchReplyHere *reply = new QPlaceSearchReplyHere(query, 0, this);
             connect(reply, SIGNAL(finished()), this, SLOT(replyFinished()));
             connect(reply, SIGNAL(error(QPlaceReply::Error,QString)),
                     this, SLOT(replyError(QPlaceReply::Error,QString)));
@@ -432,22 +403,19 @@ QPlaceSearchReply *QPlaceManagerEngineNokiaV2::search(const QPlaceSearchRequest
         if (query.limit() > 0)
             queryItems.addQueryItem(QStringLiteral("size"), QString::number(query.limit()));
 
-        if (query.offset() > -1)
-            queryItems.addQueryItem(QStringLiteral("offset"), QString::number(query.offset()));
-
         u.setQuery(queryItems);
 
         networkReply = sendRequest(u);
     } else if (!query.searchTerm().isEmpty()) {
         // search term query
         QUrl requestUrl(QString::fromLatin1("http://") + m_uriProvider->getCurrentHost() +
-                        QLatin1String("/places/v1/discover/search"));
+                        QStringLiteral("/places/v1/discover/search"));
 
-        queryItems.addQueryItem(QLatin1String("q"), query.searchTerm());
-        queryItems.addQueryItem(QLatin1String("tf"), QLatin1String("html"));
+        queryItems.addQueryItem(QStringLiteral("q"), query.searchTerm());
+        queryItems.addQueryItem(QStringLiteral("tf"), QStringLiteral("html"));
 
         if (query.limit() > 0) {
-            queryItems.addQueryItem(QLatin1String("size"),
+            queryItems.addQueryItem(QStringLiteral("size"),
                                     QString::number(query.limit()));
         }
 
@@ -455,7 +423,7 @@ QPlaceSearchReply *QPlaceManagerEngineNokiaV2::search(const QPlaceSearchRequest
 
         QNetworkReply *networkReply = sendRequest(requestUrl);
 
-        QPlaceSearchReplyImpl *reply = new QPlaceSearchReplyImpl(query, networkReply, this);
+        QPlaceSearchReplyHere *reply = new QPlaceSearchReplyHere(query, networkReply, this);
         connect(reply, SIGNAL(finished()), this, SLOT(replyFinished()));
         connect(reply, SIGNAL(error(QPlaceReply::Error,QString)),
                 this, SLOT(replyError(QPlaceReply::Error,QString)));
@@ -463,10 +431,10 @@ QPlaceSearchReply *QPlaceManagerEngineNokiaV2::search(const QPlaceSearchRequest
         return reply;
     } else if (!query.recommendationId().isEmpty()) {
         QUrl requestUrl(QString::fromLatin1("http://") + m_uriProvider->getCurrentHost() +
-                        QLatin1String("/places/v1/places/") + query.recommendationId() +
-                        QLatin1String("/related/recommended"));
+                        QStringLiteral("/places/v1/places/") + query.recommendationId() +
+                        QStringLiteral("/related/recommended"));
 
-        queryItems.addQueryItem(QLatin1String("tf"), QLatin1String("html"));
+        queryItems.addQueryItem(QStringLiteral("tf"), QStringLiteral("html"));
 
         requestUrl.setQuery(queryItems);
 
@@ -487,23 +455,19 @@ QPlaceSearchReply *QPlaceManagerEngineNokiaV2::search(const QPlaceSearchRequest
 
         addAtForBoundingArea(query.searchArea(), &queryItems);
 
-        queryItems.addQueryItem(QLatin1String("tf"), QLatin1String("html"));
+        queryItems.addQueryItem(QStringLiteral("tf"), QStringLiteral("html"));
 
         if (query.limit() > 0) {
-            queryItems.addQueryItem(QLatin1String("size"),
+            queryItems.addQueryItem(QStringLiteral("size"),
                                     QString::number(query.limit()));
         }
-        if (query.offset() > -1) {
-            queryItems.addQueryItem(QLatin1String("offset"),
-                                    QString::number(query.offset()));
-        }
 
         requestUrl.setQuery(queryItems);
 
         networkReply = sendRequest(requestUrl);
     }
 
-    QPlaceSearchReplyImpl *reply = new QPlaceSearchReplyImpl(query, networkReply, this);
+    QPlaceSearchReplyHere *reply = new QPlaceSearchReplyHere(query, networkReply, this);
     connect(reply, SIGNAL(finished()), this, SLOT(replyFinished()));
     connect(reply, SIGNAL(error(QPlaceReply::Error,QString)),
             this, SLOT(replyError(QPlaceReply::Error,QString)));
@@ -518,7 +482,6 @@ QPlaceSearchSuggestionReply *QPlaceManagerEngineNokiaV2::searchSuggestions(const
     unsupported |= query.visibilityScope() != QLocation::UnspecifiedVisibility &&
                    query.visibilityScope() != QLocation::PublicVisibility;
 
-    unsupported |= query.offset() > 0;
     unsupported |= !query.categories().isEmpty();
     unsupported |= !query.recommendationId().isEmpty();
 
@@ -534,11 +497,11 @@ QPlaceSearchSuggestionReply *QPlaceManagerEngineNokiaV2::searchSuggestions(const
     }
 
     QUrl requestUrl(QString::fromLatin1("http://") + m_uriProvider->getCurrentHost() +
-                    QLatin1String("/places/v1/suggest"));
+                    QStringLiteral("/places/v1/suggest"));
 
     QUrlQuery queryItems;
 
-    queryItems.addQueryItem(QLatin1String("q"), query.searchTerm());
+    queryItems.addQueryItem(QStringLiteral("q"), query.searchTerm());
 
     if (!addAtForBoundingArea(query.searchArea(), &queryItems)) {
         QPlaceSearchSuggestionReplyImpl *reply = new QPlaceSearchSuggestionReplyImpl(0, this);
@@ -622,40 +585,63 @@ QPlaceReply *QPlaceManagerEngineNokiaV2::initializeCategories()
     if (m_categoryReply)
         return m_categoryReply.data();
 
-        m_tempTree.clear();
-        CategoryParser parser;
+    m_tempTree.clear();
+    CategoryParser parser;
 
-        if (!parser.parse(m_localDataPath + QLatin1String("/offline/offline-mapping.json"))) {
-            PlaceCategoryNode rootNode;
+    if (parser.parse(m_localDataPath + QStringLiteral("/offline/offline-mapping.json"))) {
+        m_tempTree = parser.tree();
+    } else {
+        PlaceCategoryNode rootNode;
 
-            for (int i = 0; FIXED_CATEGORIES_indices[i] != -1; ++i) {
-                const QString id = QString::fromLatin1(FIXED_CATEGORIES_string +
-                                                       FIXED_CATEGORIES_indices[i]);
-                m_tempTree.insert(id, PlaceCategoryNode());
+        for (int i = 0; FIXED_CATEGORIES_indices[i] != -1; ++i) {
+            const QString id = QString::fromLatin1(FIXED_CATEGORIES_string +
+                                                   FIXED_CATEGORIES_indices[i]);
+
+            int subCatDivider = id.indexOf(QChar('|'));
+            if (subCatDivider >= 0) {
+                // found a sub category
+                const QString subCategoryId = id.mid(subCatDivider+1);
+                const QString parentCategoryId = id.left(subCatDivider);
+
+                if (m_tempTree.contains(parentCategoryId)) {
+                    PlaceCategoryNode node;
+                    node.category.setCategoryId(subCategoryId);
+                    node.parentId = parentCategoryId;
+
+                    // find parent
+                    PlaceCategoryNode &parent = m_tempTree[parentCategoryId];
+                    parent.childIds.append(subCategoryId);
+                    m_tempTree.insert(subCategoryId, node);
+                }
+
+            } else {
+                PlaceCategoryNode node;
+                node.category.setCategoryId(id);
+
+                m_tempTree.insert(id, node);
                 rootNode.childIds.append(id);
             }
-
-            m_tempTree.insert(QString(), rootNode);
-        } else {
-            m_tempTree = parser.tree();
         }
 
-        //request all categories in the tree from the server
-        //because we don't want the root node, we remove it from the list
-        QStringList ids = m_tempTree.keys();
-        ids.removeAll(QString());
-        foreach (const QString &id, ids) {
-            QUrl requestUrl(QString::fromLatin1("http://") + m_uriProvider->getCurrentHost() +
-                            QLatin1String("/places/v1/categories/places/") + id);
-            QNetworkReply *networkReply = sendRequest(requestUrl);
-            connect(networkReply, SIGNAL(finished()), this, SLOT(categoryReplyFinished()));
-            connect(networkReply, SIGNAL(error(QNetworkReply::NetworkError)),
-                    this, SLOT(categoryReplyError()));
+        m_tempTree.insert(QString(), rootNode);
+    }
 
-            m_categoryRequests.insert(id, networkReply);
-        }
+    //request all categories in the tree from the server
+    //because we don't want the root node, we remove it from the list
+    QStringList ids = m_tempTree.keys();
+    ids.removeAll(QString());
+    foreach (const QString &id, ids) {
+        QUrl requestUrl(QString::fromLatin1("http://") + m_uriProvider->getCurrentHost() +
+                        QStringLiteral("/places/v1/categories/places/") + id);
+        QNetworkReply *networkReply = sendRequest(requestUrl);
+        connect(networkReply, SIGNAL(finished()), this, SLOT(categoryReplyFinished()));
+        connect(networkReply, SIGNAL(error(QNetworkReply::NetworkError)),
+                this, SLOT(categoryReplyError()));
 
-    QPlaceCategoriesReplyImpl *reply = new QPlaceCategoriesReplyImpl(this);
+        m_categoryRequests.insert(id, networkReply);
+    }
+
+    QPlaceCategoriesReplyHere *reply = new QPlaceCategoriesReplyHere(this);
     connect(reply, SIGNAL(finished()), this, SLOT(replyFinished()));
     connect(reply, SIGNAL(error(QPlaceReply::Error,QString)),
             this, SLOT(replyError(QPlaceReply::Error,QString)));
@@ -786,6 +772,8 @@ void QPlaceManagerEngineNokiaV2::categoryReplyFinished()
     if (!reply)
         return;
 
+    QString categoryId;
+
     if (reply->error() == QNetworkReply::NoError) {
         QJsonDocument document = QJsonDocument::fromJson(reply->readAll());
         if (!document.isObject()) {
@@ -799,23 +787,32 @@ void QPlaceManagerEngineNokiaV2::categoryReplyFinished()
 
         QJsonObject category = document.object();
 
-        QString categoryId = category.value(QLatin1String("categoryId")).toString();
+        categoryId = category.value(QStringLiteral("categoryId")).toString();
         if (m_tempTree.contains(categoryId)) {
             PlaceCategoryNode node = m_tempTree.value(categoryId);
-            node.category.setName(category.value(QLatin1String("name")).toString());
+            node.category.setName(category.value(QStringLiteral("name")).toString());
             node.category.setCategoryId(categoryId);
-            node.category.setIcon(icon(category.value(QLatin1String("icon")).toString()));
+            node.category.setIcon(icon(category.value(QStringLiteral("icon")).toString()));
 
             m_tempTree.insert(categoryId, node);
         }
+    } else {
+        categoryId = m_categoryRequests.key(reply);
+        PlaceCategoryNode rootNode = m_tempTree.value(QString());
+        rootNode.childIds.removeAll(categoryId);
+        m_tempTree.insert(QString(), rootNode);
+        m_tempTree.remove(categoryId);
     }
 
-    m_categoryRequests.remove(m_categoryRequests.key(reply));
+    m_categoryRequests.remove(categoryId);
     reply->deleteLater();
 
-    if (m_categoryRequests.isEmpty() && m_categoryReply) {
+    if (m_categoryRequests.isEmpty()) {
         m_categoryTree = m_tempTree;
-        m_categoryReply.data()->emitFinished();
+        m_tempTree.clear();
+
+        if (m_categoryReply)
+            m_categoryReply.data()->emitFinished();
     }
 }
 
@@ -831,8 +828,8 @@ void QPlaceManagerEngineNokiaV2::categoryReplyError()
 QNetworkReply *QPlaceManagerEngineNokiaV2::sendRequest(const QUrl &url)
 {
     QUrlQuery queryItems(url);
-    queryItems.addQueryItem(QLatin1String("app_id"), m_appId);
-    queryItems.addQueryItem(QLatin1String("app_code"), m_appCode);
+    queryItems.addQueryItem(QStringLiteral("app_id"), m_appId);
+    queryItems.addQueryItem(QStringLiteral("app_code"), m_appCode);
 
     QUrl requestUrl = url;
     requestUrl.setQuery(queryItems);
